#!/usr/bin/perl -w

# $Id: $
# Converts a font so it can used on a microcontroller to draw it on a
# graphical LCD display

sub showhelpandexit() {
  print("Syntax: $0 textfontfile\n");
  print("Output goes to STDOUT.\n");
  exit(1);
}

sub printerrandexit($) {
  print($_[0]);
  exit(1);
}

# main() ---------------------
unless (@ARGV == 1) {
  showhelpandexit();
}
unless (open($SRCFILE, '<'.$ARGV[0])) {
  printerrandexit("Failed to open $ARGV[0]\n");
}
$curchar = undef;
$lineno = 0;
@chardata = ();
@charlines = ();
@charwidth = ();
while ($lastl = <$SRCFILE>) {
  $lastl =~ s/[\r\n]//g;
  $lineno++;
  if ($lastl =~ m/^\s*#/) { next; } # Ignore comments
  if ($lastl =~ m/^\s*$/) { next; } # Ignore empty lines
  if ($lastl =~ m/^\:(.)/) {
    $curchar = ord($1);
    if ($lastl =~ m/^\:DEFAULT/) { $curchar = 0; }
    if ($lastl =~ m/^\:#(\d+)/) { $curchar = int($1); }
    if ($lastl =~ m/^\:#0x([a-fA-F0-9]+)/) { $curchar = hex($1); }
    if ((defined($charlines[$curchar]))
     && ($charlines[$curchar] > 0)) {
      print(STDERR "WARNING: redefining character with ASCII code $curchar ('".chr($curchar)."')\n");
    }
    $charlines[$curchar] = 0;
    $charwidth[$curchar] = 0;
  } else {
    # Try to remove linenumbers (will fail if there aren't any, but doesn't hurt).
    unless ($lastl =~ s/^\d //) {
      $lastl =~ s/^\d//;
    }
    unless (defined($curchar)) {
      print(STDERR "WARNING: Line with data not belonging to a character (line $lineno)");
    }
    $chardata[$curchar][$charlines[$curchar]] = $lastl;
    $charlines[$curchar]++;
    if (length($lastl) > $charwidth[$curchar]) {
      $charwidth[$curchar] = length($lastl);
    }
  }
}
close($SRCFILE);
# Create dummy character #0 (the 'DEFAULT') if it does not exist
unless ((defined($charlines[0]))
     && ($charlines[0] > 0)) {
  $charlines[0] = 1;
  $charwidth[0] = 1;
  $chardata[0][0] = ' ';
}
$chardefout = '';
$fontmaxwidth = 0; $fontheight = 0;
$currentoffset = 0;
@charoffsets = ();
for ($reali = 0; $reali < 256; $reali++) {
  if ((defined($charlines[$reali]))
   && ($charlines[$reali] > 0)) {
    $i = $reali;
  } else {
    # No definition for this character
    # fallback: Char #0
    $i = 0;
  }
  if ($charlines[$i] > 8) {
    print(STDERR "WARNING: Character with ASCII code $i ('".chr($i)."') is defined with a height of more than 8 and will be truncated.\n");
  }
  if ($charlines[$i] > $fontheight) { $fontheight = $charlines[$i]; }
  if ($charwidth[$i] > $fontmaxwidth) { $fontmaxwidth = $charwidth[$i]; }
  if (($reali > 32) && ($reali < 127)) {
    $chardefout .= sprintf("  /* #%03d / 0x%02x ('%c') */ ", $reali, $reali, $reali);
  } else {
    $chardefout .= sprintf("  /* #%03d / 0x%02x       */ ", $reali, $reali);
  }
  for ($j = 0; $j < $charwidth[$i]; $j++) { # x
    $curbyte = 0;
    for ($k = 0; $k < 8; $k++) { # y
      if (($charlines[$i] > $k)
       && (length($chardata[$i][$k]) > $j)) {
        $cc = substr($chardata[$i][$k], $j, 1);
      } else {
        $cc = ' ';
      }
      if ((defined($cc)) && ($cc ne ' ')) {
        $curbyte |= (1 << $k);
      }
    }
    $chardefout .= sprintf('0x%02x, ', $curbyte);
    $currentoffset++;
  }
  $charoffsets[$reali] = $currentoffset;
  if ($curbyte == 0) {
    print(STDERR "WARNING: Character with ASCII code $i ('".chr($i)."') ends with empty row.\n");
  }
  $chardefout .= "\n";
}
print(STDERR "Here is your font definition:\n");
print("static prog_uint8_t lcd_deffontdata[] = {\n");
print($chardefout);
print("};\n");
print("static prog_uint16_t lcd_deffontoffsets[] = {\n      ");
for ($i = 0; $i < 256; $i++) {
  print($charoffsets[$i]);
  if ($i != 255) { print(", "); }
  if (($i % 16) == 15) {
    print("\n      ");
  }
}
print("};\n");
print("struct lcd_font lcd_defaultfont = {\n");
print("  .maxwidth = $fontmaxwidth,\n");
print("  .height = $fontheight,\n");
print("  .offsets = lcd_deffontoffsets,\n");
print("  .data = lcd_deffontdata\n");
print("};\n");
