#!/usr/bin/perl -w

# $Id: $
# Converts a (black and white) image so it can used on a microcontroller to
# draw it on a graphical LCD display.
# Transparency is supported, use the parameter --transparent.
# libgd and the corresponding perl module are needed.

use GD;

$fn = ''; $transparent = 0; $invert = 0;
for ($i = 0; $i < @ARGV; $i++) {
  if ($ARGV[$i] eq '--transparent') {
    $transparent = 1;
  } elsif ($ARGV[$i] eq '--invert') {
    $invert = 1;
  } else {
    if (length($fn) > 0) {
      print(STDERR "Syntax: $0 [--transparent] [--invert] filename\n");
      exit(1);
    }
    $fn = $ARGV[$i];
  }
}
if (length($fn) < 1) {
  print(STDERR "ERROR: No file to convert given.\n");
  exit(1);
}
$im = new GD::Image($fn);
unless (defined($im)) {
  print(STDERR "ERROR: Something went wrong - image could not be loaded.\n");
  exit(1);
}
if (($im->width < 1) || ($im->height < 1)) {
  print(STDERR "ERROR: Image has a size of 0.\n");
  exit(1);
}
if (($im->width > 128) || ($im->height > 64)) {
  print(STDERR "WARNING: The image has size ".int($im->width)." x ".int($im->height));
  print(STDERR " which is larger than the 128 x 64 of a normal graphical LCD\n");
}
if ($transparent) {
  $tridx = $im->transparent();
  if ((!defined($tridx)) || ($tridx < 0)) {
    print(STDERR "ERROR: transparency requested, but Image has no transparency!\n");
    exit(1);
  } else {
    print(STDERR "Transparency index: $tridx\n");
  }
}
$curibyte = 0; $curtbyte = 0;
@ibytes = ();  @tbytes = ();
$curbit = 0;
for ($y = 0; $y < $im->height; $y++) {
  for ($x = 0; $x < $im->width; $x++) {
    $idx = $im->getPixel($x, $y);
    if (($transparent) && ($idx == $tridx)) {
      print(STDERR ".");
      $curtbyte |= (1 << $curbit);
    } else {
      ($r, $g, $b) = $im->rgb($idx);
      if (($r >= 128) || ($g >= 128) || ($b >= 128)) {
        if ($invert) {
          print(STDERR "X");
          $curibyte |= (1 << $curbit);
        } else {
          print(STDERR " ");
        }
      } else {
        if ($invert) {
          print(STDERR " ");
        } else {
          print(STDERR "X");
          $curibyte |= (1 << $curbit);
        }
      }
    }
    if ($curbit == 7) { # Byte fully filled
      # Save the byte and start a new one
      push(@ibytes, $curibyte);
      push(@tbytes, $curtbyte);
      $curbit = 0;
      $curibyte = 0;
      $curtbyte = 0;
    } else {
      $curbit++;
    }
  }
  print(STDERR "\n");
}
if ($curbit != 0) { # save the last half-filled byte
  push(@ibytes, $curibyte);
  push(@tbytes, $curtbyte);
}
print(STDERR "Here comes your image definition:\n");
print("static prog_uint8_t lcd_image_MYIMAGE[] = {\n");
print("    ".$im->width.", ".$im->height.", /* width, height */\n");
print("    ".(($transparent == 1) ? "1" : "0").", /* flags. 1 = has transparency */\n    ");
for ($i = 0; $i < @ibytes; $i++) {
  printf("0x%02x", $ibytes[$i]);
  if ($transparent) {
    printf(", 0x%02x", $tbytes[$i]);
  }
  if (($i+1) != @ibytes) {
    print(", ");
  }
  if (($i % 8) == 7) {
    print("/* binary image data - ".int($i+1)." bytes so far */\n    ");
  }
}
print("\n};\n");
# $curibyte = 0; $curtbyte = 0; $curbit = 0; $curidx = 0;
# for ($y = 0; $y < $im->height; $y++) {
#   for ($x = 0; $x < $im->width; $x++) {
#     if ($curbit == 0) {
#       $curibyte = $ibytes[$curidx];
#       $curtbyte = $tbytes[$curidx];
#       $curidx++;
#       #printf("[new byte %02x %02x]", $curibyte, $curtbyte);
#     }
#     if (($transparent) && ($curtbyte & 0x01)) {
#       print(".");
#     } else {
#       if ($curibyte & 0x01) {
#         print("X");
#       } else {
#         print(" ");
#       }
#     }
#     $curibyte >>= 1;
#     $curtbyte >>= 1;
#     $curbit++;
#     if ($curbit >= 8) { $curbit = 0; }
#   }
#   print("\n");
# }
